@@ -1,25 +1,25 @@
 """Browser management with Playwright for HAR recording."""
 
+import asyncio
 import io
+import json
 import logging
 import os
-
-import asyncio
-import json
 import random
 import signal
 import sys
 from contextlib import redirect_stderr
 from pathlib import Path
-from typing import Optional
 
-from playwright.sync_api import sync_playwright, Browser, BrowserContext, Page
+from playwright.sync_api import Browser, BrowserContext, Page, sync_playwright
 from playwright_stealth import Stealth
 from rich.console import Console
 from rich.status import Status
 
-from .utils import get_har_dir, get_timestamp
 from . import __version__
+from .action_recorder import ActionRecorder, RecordedAction
+from .tui import ERROR_CTA
+from .utils import get_har_dir, get_timestamp
 
 console = Console()
 
@@ -203,21 +203,211 @@ class ManualBrowser:
         prompt: str,
         output_dir: str | None = None,
         use_real_chrome: bool = True,  # New option to use real Chrome
+        enable_action_recording: bool = False,
     ):
         self.run_id = run_id
         self.prompt = prompt
         self.output_dir = output_dir
         self.use_real_chrome = use_real_chrome
+        self.enable_action_recording = enable_action_recording
+        
         self.har_dir = get_har_dir(run_id, output_dir)
         self.har_path = self.har_dir / "recording.har"
         self.metadata_path = self.har_dir / "metadata.json"
+        
         self._playwright = None
-        self._browser: Optional[Browser] = None
-        self._context: Optional[BrowserContext] = None
-        self._start_time: Optional[str] = None
+        self._browser: Browser | None = None
+        self._context: BrowserContext | None = None
+        self._start_time: str | None = None
         self._user_agent = random.choice(USER_AGENTS)
         self._using_persistent = False  # Track if using persistent context
 
+        self.action_recorder = ActionRecorder() if enable_action_recording else None
+
+    def _inject_action_recorder(self, page: Page) -> None:
+        """Inject action recording script into page.
+        
+        Uses console.log + page.on('console') for reliable capture.
+        Works best with stealth Chromium mode (not real Chrome).
+        """
+        if not self.enable_action_recording:
+            return
+        
+        # Simple JS that logs actions to console with a special prefix
+        recorder_js = '''
+        window.__recordedActions = [];
+        window.__lastUrl = null;
+        
+        document.addEventListener('click', (e) => {
+            const el = e.target;
+            
+            // Build a robust selector by traversing up to find parent with ID
+            function buildSelector(element) {
+                // Priority 1: data-testid on element or close parent
+                let current = element;
+                for (let i = 0; i < 3 && current; i++) {
+                    if (current.dataset && current.dataset.testid) {
+                        const path = i === 0 ? '' : ' ' + getPathFromAncestor(current, element);
+                        return '[data-testid="' + current.dataset.testid.replace(/"/g, '\\"') + '"]' + path;
+                    }
+                    current = current.parentElement;
+                }
+                
+                // Priority 2: element has short ID
+                if (element.id && element.id.length < 20) {
+                    return '#' + element.id;
+                }
+                
+                // Priority 3: find parent with ID and build path
+                current = element.parentElement;
+                let depth = 1;
+                while (current && depth < 5) {
+                    if (current.id && current.id.length < 20) {
+                        const path = getPathFromAncestor(current, element);
+                        const selector = '#' + current.id + ' > ' + path;
+                        // Verify it's unique
+                        if (document.querySelectorAll(selector).length === 1) {
+                            return selector;
+                        }
+                    }
+                    current = current.parentElement;
+                    depth++;
+                }
+                
+                // Priority 4: name attribute
+                if (element.name) {
+                    return '[name="' + element.name.replace(/"/g, '\\"') + '"]';
+                }
+                
+                // Priority 5: aria-label
+                if (element.getAttribute && element.getAttribute('aria-label')) {
+                    return '[aria-label="' + element.getAttribute('aria-label').replace(/"/g, '\\"') + '"]';
+                }
+                
+                // Priority 6: role + text for buttons
+                if ((element.tagName === 'BUTTON' || element.role === 'button') && element.innerText) {
+                    const text = element.innerText.trim().substring(0, 30);
+                    if (text && !text.includes('\\n')) {
+                        return 'button:has-text(' + JSON.stringify(text) + ')';
+                    }
+                }
+                
+                // Priority 7: link text
+                if (element.tagName === 'A' && element.innerText) {
+                    const text = element.innerText.trim().substring(0, 30);
+                    if (text && !text.includes('\\n')) {
+                        return 'a:has-text(' + JSON.stringify(text) + ')';
+                    }
+                }
+                
+                // Priority 8: tag + class
+                if (element.className && typeof element.className === 'string') {
+                    const cls = element.className.split(' ').filter(c => c && c.length < 30 && !c.includes('hover') && !c.includes('active'))[0];
+                    if (cls) {
+                        const baseSelector = element.tagName.toLowerCase() + '.' + cls;
+                        const matches = document.querySelectorAll(baseSelector);
+                        if (matches.length === 1) return baseSelector;
+                        const idx = Array.from(matches).indexOf(element);
+                        if (idx >= 0) return baseSelector + ' >> nth=' + idx;
+                    }
+                }
+                
+                // Fallback: tag name
+                return element.tagName.toLowerCase();
+            }
+            
+            // Get path from ancestor to descendant
+            function getPathFromAncestor(ancestor, descendant) {
+                if (ancestor === descendant) return descendant.tagName.toLowerCase();
+                
+                let path = [];
+                let current = descendant;
+                while (current && current !== ancestor) {
+                    path.unshift(current.tagName.toLowerCase());
+                    current = current.parentElement;
+                }
+                return path.join(' > ');
+            }
+            
+            const selector = buildSelector(el);
+            const action = {type: 'click', selector: selector, timestamp: Date.now()};
+            window.__recordedActions.push(action);
+            console.log('__ACTION__' + JSON.stringify(action));
+        }, true);
+        
+        document.addEventListener('input', (e) => {
+            if (e.target.tagName === 'INPUT' || e.target.tagName === 'TEXTAREA') {
+                const el = e.target;
+                let selector = '';
+                if (el.id && el.id.length < 20) selector = '#' + el.id;
+                else if (el.name) selector = '[name="' + el.name.replace(/"/g, '\\"') + '"]';
+                else if (el.placeholder) selector = '[placeholder="' + el.placeholder.replace(/"/g, '\\"') + '"]';
+                else selector = el.tagName.toLowerCase();
+                
+                const action = {type: 'fill', selector: selector, value: el.value, timestamp: Date.now()};
+                window.__recordedActions.push(action);
+                console.log('__ACTION__' + JSON.stringify(action));
+            }
+        }, true);
+        
+        document.addEventListener('keydown', (e) => {
+            if (e.key === 'Enter') {
+                const el = e.target;
+                let selector = '';
+                if (el.id && el.id.length < 20) selector = '#' + el.id;
+                else if (el.name) selector = '[name="' + el.name.replace(/"/g, '\\"') + '"]';
+                else selector = el.tagName.toLowerCase();
+                
+                const action = {type: 'press', selector: selector, value: 'Enter', timestamp: Date.now()};
+                window.__recordedActions.push(action);
+                console.log('__ACTION__' + JSON.stringify(action));
+            }
+        }, true);
+        
+        // Only log navigation for top-level main frame, not iframes
+        if (window === window.top) {
+            const url = window.location.href;
+            // Skip about:blank, blob:, data:, service workers, embeds
+            if (url && !url.startsWith('about:') && !url.startsWith('blob:') && 
+                !url.startsWith('data:') && !url.includes('/embed') && 
+                !url.includes('service_worker') && !url.includes('googletagmanager')) {
+                // Only log if URL changed
+                if (url !== window.__lastUrl) {
+                    window.__lastUrl = url;
+                    console.log('__ACTION__' + JSON.stringify({type: 'navigate', url: url, timestamp: Date.now()}));
+                }
+            }
+        }
+        '''
+        
+        # Listen to console for actions
+        import json
+        last_url = [None]  # Mutable to track last URL
+        
+        def on_console(msg):
+            text = msg.text
+            if text.startswith('__ACTION__'):
+                try:
+                    action_json = text[10:]  # Remove '__ACTION__' prefix
+                    action_data = json.loads(action_json)
+                    
+                    # Filter duplicate navigations
+                    if action_data.get('type') == 'navigate':
+                        url = action_data.get('url', '')
+                        if url == last_url[0]:
+                            return  # Skip duplicate
+                        last_url[0] = url
+                    
+                    if self.action_recorder:
+                        self.action_recorder.add_action(RecordedAction(**action_data))
+                except Exception as e:
+                    console.print(f" [dim]action parse error: {e}[/dim]")
+        
+        page.on("console", on_console)
+        page.add_init_script(recorder_js)
+        
+        console.print(" [dim]action recording enabled[/dim]")
+
     def _save_metadata(self, end_time: str) -> None:
         """Save run metadata to JSON file."""
         metadata = {
@@ -232,7 +422,7 @@ class ManualBrowser:
 
     def _handle_signal(self, signum, frame) -> None:
         """Handle interrupt signals gracefully."""
-        console.print(f"\n\n [dim]terminating capture...[/dim]")
+        console.print("\n\n [dim]terminating capture...[/dim]")
         self.close()
         sys.exit(0)
 
@@ -240,24 +430,22 @@ class ManualBrowser:
         """Inject stealth scripts into page before any other scripts run."""
         page.add_init_script(STEALTH_JS)
 
-    def _start_with_real_chrome(self, start_url: Optional[str] = None) -> Path:
+    def _start_with_real_chrome(self, start_url: str | None = None) -> Path:
         """Start using the real Chrome browser with user's profile."""
         import shutil
         import tempfile
 
         chrome_profile = get_chrome_profile_dir()
         if not chrome_profile:
-            console.print(
-                f" [yellow]chrome profile not found, falling back to stealth mode[/yellow]"
-            )
+            console.print(" [yellow]chrome profile not found, falling back to stealth mode[/yellow]")
             return self._start_with_stealth_chromium(start_url)
 
         # Create a temporary profile directory
         temp_profile_dir = Path(tempfile.mkdtemp(prefix="chrome_profile_"))
 
-        console.print(f" [dim]using real chrome (profile copy)[/dim]")
-        console.print(f" [yellow]⚠️  please browse in the FIRST tab only[/yellow]")
-        console.print(f" [yellow]    (new tabs may not be recorded)[/yellow]")
+        console.print(" [dim]using real chrome (profile copy)[/dim]")
+        console.print(" [yellow]⚠️  please browse in the FIRST tab only[/yellow]")
+        console.print(" [yellow]    (new tabs may not be recorded)[/yellow]")
         console.print()
 
         try:
@@ -286,6 +474,9 @@ class ManualBrowser:
             # For HAR recording & context
             page = self._context.new_page()
 
+            if self.enable_action_recording:
+                self._inject_action_recorder(page)
+
             if start_url:
                 page.goto(start_url, wait_until="domcontentloaded")
             else:
@@ -307,7 +498,7 @@ class ManualBrowser:
             except Exception:
                 pass
 
-    def _start_with_stealth_chromium(self, start_url: Optional[str] = None) -> Path:
+    def _start_with_stealth_chromium(self, start_url: str | None = None) -> Path:
         """Start using Playwright's Chromium with stealth patches."""
         # Comprehensive stealth Chrome arguments
         chrome_args = [
@@ -378,6 +569,9 @@ class ManualBrowser:
         # Open initial page
         page = self._context.new_page()
 
+        if self.enable_action_recording:
+            self._inject_action_recorder(page)
+
         if start_url:
             page.goto(start_url, wait_until="domcontentloaded")
         else:
@@ -393,7 +587,7 @@ class ManualBrowser:
 
         return self.close()
 
-    def start(self, start_url: Optional[str] = None) -> Path:
+    def start(self, start_url: str | None = None) -> Path:
         """Start the browser with HAR recording enabled. Returns HAR path when done."""
         self._start_time = get_timestamp()
 
@@ -401,17 +595,18 @@ class ManualBrowser:
         signal.signal(signal.SIGINT, self._handle_signal)
         signal.signal(signal.SIGTERM, self._handle_signal)
 
-        console.print(f" [dim]capture starting...[/dim]")
+        console.print(" [dim]capture starting...[/dim]")
         console.print(f" [dim]━[/dim] [white]{self.run_id}[/white]")
         console.print(f" [dim]goal[/dim]  [white]{self.prompt}[/white]")
         console.print()
-        console.print(f" [dim]navigate and interact to record traffic[/dim]")
-        console.print(f" [dim]close browser or ctrl+c to finalize[/dim]")
+        console.print(" [dim]navigate and interact to record traffic[/dim]")
+        console.print(" [dim]close browser or ctrl+c to finalize[/dim]")
         console.print()
 
         self._playwright = sync_playwright().start()
 
-        # Try real Chrome first, fall back to stealth Chromium
+        # Try real Chrome first (better for avoiding detection)
+        # Fall back to stealth Chromium if Chrome not available
         if self.use_real_chrome:
             return self._start_with_real_chrome(start_url)
         else:
@@ -421,7 +616,7 @@ class ManualBrowser:
         """Close the browser and save HAR file. Returns HAR path."""
         end_time = get_timestamp()
 
-        console.print(f" [dim]browser closed[/dim]")
+        console.print(" [dim]browser closed[/dim]")
 
         if self._context:
             with Status(
@@ -442,14 +637,12 @@ class ManualBrowser:
                         har_size = self.har_path.stat().st_size
                         status.update(f" [dim]har saved: {har_size:,} bytes[/dim]")
                     else:
-                        console.print(
-                            " [yellow]warning: har file was not created[/yellow]"
-                        )
+                        console.print(" [yellow]warning: har file was not created[/yellow]")
 
                 except Exception as e:
                     console.print(f" [yellow]warning: error saving har: {e}[/yellow]")
                     if self.har_path.exists():
-                        console.print(f" [dim]har file exists despite error[/dim]")
+                        console.print(" [dim]har file exists despite error[/dim]")
                 self._context = None
 
         # Only close browser if not using persistent context
@@ -470,15 +663,21 @@ class ManualBrowser:
         # Save metadata
         self._save_metadata(end_time)
 
-        console.print(f" [dim]capture saved[/dim]")
-        console.print(f" [dim]metadata synced[/dim]")
+        console.print(" [dim]capture saved[/dim]")
+        console.print(" [dim]metadata synced[/dim]")
+
+        if self.action_recorder:
+            try:
+                actions_path = self.har_dir / "actions.json"
+                self.action_recorder.save(actions_path)
+                console.print(" [dim]actions saved[/dim]")
+            except Exception as e:
+                console.print(f" [yellow]warning: error saving actions: {e}[/yellow]")
 
         return self.har_path
 
 
-def parse_agent_model(
-    agent_model: str, agent_provider: str = "browser-use"
-) -> tuple[str, str | None]:
+def parse_agent_model(agent_model: str, agent_provider: str = "browser-use") -> tuple[str, str | None]:
     """Parse agent_model string into provider and model name.
 
     Args:
@@ -515,15 +714,11 @@ def parse_agent_model(
 
                 if provider == "openai":
                     if model_name not in openai_cua_models:
-                        raise ValueError(
-                            f"Unsupported OpenAI model: {model_name}. "
-                            f"Supported OpenAI CUA models: {', '.join(openai_cua_models)}"
-                        )
+                        raise ValueError(f"Unsupported OpenAI model: {model_name}. Supported OpenAI CUA models: {', '.join(openai_cua_models)}")
                 elif provider == "anthropic":
                     if model_name not in anthropic_cua_models:
                         raise ValueError(
-                            f"Unsupported Anthropic model: {model_name}. "
-                            f"Supported Anthropic CUA models: {', '.join(anthropic_cua_models)}"
+                            f"Unsupported Anthropic model: {model_name}. Supported Anthropic CUA models: {', '.join(anthropic_cua_models)}"
                         )
                 else:
                     raise ValueError(
@@ -533,15 +728,10 @@ def parse_agent_model(
 
             return (provider, model_name)
 
-    raise ValueError(
-        f"Invalid agent_model format: {agent_model}. "
-        f"Expected 'bu-llm' or '{{provider}}/{{model_name}}' (e.g., 'openai/gpt-4')"
-    )
+    raise ValueError(f"Invalid agent_model format: {agent_model}. Expected 'bu-llm' or '{{provider}}/{{model_name}}' (e.g., 'openai/gpt-4')")
 
 
-def get_required_api_key(
-    provider: str, agent_provider: str = "browser-use"
-) -> tuple[str, str]:
+def get_required_api_key(provider: str, agent_provider: str = "browser-use") -> tuple[str, str]:
     """Get the required API key environment variable name for a provider.
 
     Args:
@@ -569,16 +759,12 @@ def get_required_api_key(
 
     if provider not in provider_map:
         supported = ", ".join(provider_map.keys())
-        raise ValueError(
-            f"Unsupported provider: {provider}. Supported providers: {supported}"
-        )
+        raise ValueError(f"Unsupported provider: {provider}. Supported providers: {supported}")
 
     return provider_map[provider]
 
 
-def validate_api_key(
-    provider: str, agent_provider: str = "browser-use"
-) -> tuple[bool, str | None]:
+def validate_api_key(provider: str, agent_provider: str = "browser-use") -> tuple[bool, str | None]:
     """Validate that the required API key exists for the provider.
 
     Args:
@@ -620,7 +806,7 @@ class AgentBrowser:
         browser_use_model: str = "bu-llm",
         stagehand_model: str = "openai/computer-use-preview-2025-03-11",
         agent_provider: str = "browser-use",
-        start_url: Optional[str] = None,
+        start_url: str | None = None,
     ):
         """Initialize agent browser."""
         self.run_id = run_id
@@ -637,17 +823,13 @@ class AgentBrowser:
         self.har_path = self.har_dir / "recording.har"
         self.metadata_path = self.har_dir / "metadata.json"
 
-        self._start_time: Optional[str] = None
+        self._start_time: str | None = None
         self._cdp_port = 9222
 
     def _save_metadata(self, end_time: str, result: dict | None = None) -> None:
         """Save run metadata to JSON file."""
         # Select the appropriate model based on agent_provider
-        agent_model = (
-            self.stagehand_model
-            if self.agent_provider == "stagehand"
-            else self.browser_use_model
-        )
+        agent_model = self.stagehand_model if self.agent_provider == "stagehand" else self.browser_use_model
 
         metadata = {
             "run_id": self.run_id,
@@ -679,12 +861,12 @@ class AgentBrowser:
         import os
 
         # Set before importing to ensure it takes effect
-        os.environ['BROWSER_USE_LOGGING_LEVEL'] = 'WARNING'
+        os.environ["BROWSER_USE_LOGGING_LEVEL"] = "WARNING"
+        os.environ["BROWSER_USE_LOGGING_LEVEL"] = "WARNING"
 
         # Import browser-use after setting environment variable
         try:
-            from browser_use import Agent, Browser
-            from browser_use import ChatBrowserUse
+            from browser_use import Agent, Browser, ChatBrowserUse
 
             # Suppress all browser-use loggers after import
             def suppress_browser_use_logs():
@@ -709,6 +891,7 @@ class AgentBrowser:
                 "error": "browser-use is required for agent mode. Install it with: pip install git+https://github.com/browser-use/browser-use.git@49a345fb19e9f12befc5cc1658e0033873892455",
             }
             console.print(f" [red]error:[/red] {result['error']}")
+            console.print(f" [dim]{ERROR_CTA}[/dim]")
             return result
 
         result = {"success": False, "message": None, "error": None}
@@ -717,12 +900,11 @@ class AgentBrowser:
         try:
             # Parse agent model and validate API key
             try:
-                provider, model_name = parse_agent_model(
-                    self.browser_use_model, self.agent_provider
-                )
+                provider, model_name = parse_agent_model(self.browser_use_model, self.agent_provider)
             except ValueError as e:
                 result["error"] = str(e)
                 console.print(f" [red]error:[/red] {e}")
+                console.print(f" [dim]{ERROR_CTA}[/dim]")
                 return result
 
             # Validate API key
@@ -730,6 +912,7 @@ class AgentBrowser:
             if not is_valid:
                 result["error"] = error_msg
                 console.print(f" [red]error:[/red] {error_msg}")
+                console.print(f" [dim]{ERROR_CTA}[/dim]")
                 return result
 
             # Create appropriate LLM instance
@@ -746,6 +929,7 @@ class AgentBrowser:
                 except ImportError:
                     result["error"] = "Failed to initialize OpenAI LLM"
                     console.print(f" [red]error:[/red] {result['error']}")
+                    console.print(f" [dim]{ERROR_CTA}[/dim]")
                     return result
             elif provider == "google":
                 try:
@@ -755,15 +939,17 @@ class AgentBrowser:
                 except ImportError:
                     result["error"] = "Failed to initialize Google LLM"
                     console.print(f" [red]error:[/red] {result['error']}")
+                    console.print(f" [dim]{ERROR_CTA}[/dim]")
                     return result
             else:
                 result["error"] = f"Unsupported provider: {provider}"
                 console.print(f" [red]error:[/red] {result['error']}")
+                console.print(f" [dim]{ERROR_CTA}[/dim]")
                 return result
 
             suppress_browser_use_logs()
 
-            console.print(f" [dim]starting browser with har...[/dim]")
+            console.print(" [dim]starting browser with har...[/dim]")
             browser = Browser(
                 record_har_path=str(self.har_path),
                 record_har_mode="full",
@@ -776,7 +962,7 @@ class AgentBrowser:
 
             suppress_browser_use_logs()
 
-            console.print(f" [dim]browser started[/dim]")
+            console.print(" [dim]browser started[/dim]")
 
             task = self.prompt
 
@@ -807,7 +993,7 @@ class AgentBrowser:
                         har_size = self.har_path.stat().st_size
                         console.print(f" [dim]har saved: {har_size} bytes[/dim]")
                     else:
-                        console.print(f" [dim]har not saved by browser-use[/dim]")
+                        console.print(" [dim]har not saved by browser-use[/dim]")
                 except Exception as e:
                     console.print(f" [dim]browser stop error: {e}[/dim]")
 
@@ -831,21 +1017,21 @@ class AgentBrowser:
 
         try:
             try:
-                provider, model_name = parse_agent_model(
-                    self.stagehand_model, self.agent_provider
-                )
+                provider, model_name = parse_agent_model(self.stagehand_model, self.agent_provider)
             except ValueError as e:
                 result["error"] = str(e)
                 console.print(f" [red]error:[/red] {e}")
+                console.print(f" [dim]{ERROR_CTA}[/dim]")
                 return result
 
             is_valid, error_msg = validate_api_key(provider, self.agent_provider)
             if not is_valid:
                 result["error"] = error_msg
                 console.print(f" [red]error:[/red] {error_msg}")
+                console.print(f" [dim]{ERROR_CTA}[/dim]")
                 return result
 
-            console.print(f" [dim]starting stagehand with har...[/dim]")
+            console.print(" [dim]starting stagehand with har...[/dim]")
 
             env_var_name, _ = get_required_api_key(provider, self.agent_provider)
             api_key = os.getenv(env_var_name)
@@ -884,9 +1070,7 @@ class AgentBrowser:
                         not_found="abort",  # Abort requests not in HAR (but we have empty HAR)
                     )
                 except Exception as e:
-                    console.print(
-                        f" [dim]warning: could not set up HAR recording: {e}[/dim]"
-                    )
+                    console.print(f" [dim]warning: could not set up HAR recording: {e}[/dim]")
 
             agent_options = {
                 "api_key": api_key,
@@ -907,7 +1091,7 @@ class AgentBrowser:
                         options=agent_options,
                     )
 
-            console.print(f" [dim]stagehand started[/dim]")
+            console.print(" [dim]stagehand started[/dim]")
 
             # Navigate to start URL first if provided (best practice per Stagehand docs)
             if self.start_url:
@@ -922,8 +1106,7 @@ class AgentBrowser:
                 agent_result = await agent.execute(
                     {
                         "instruction": task_instruction,
-                        "maxSteps": self.timeout
-                        // 10,  # Convert timeout seconds to reasonable step limit
+                        "maxSteps": self.timeout // 10,  # Convert timeout seconds to reasonable step limit
                     }
                 )
 
@@ -937,11 +1120,7 @@ class AgentBrowser:
             if agent_result:
                 if isinstance(agent_result, dict):
                     task_success = agent_result.get("success", False)
-                    final_message = (
-                        agent_result.get("message")
-                        or agent_result.get("result")
-                        or agent_result.get("text")
-                    )
+                    final_message = agent_result.get("message") or agent_result.get("result") or agent_result.get("text")
                 elif hasattr(agent_result, "success"):
                     task_success = agent_result.success
                     # Only extract message field, ignore actions, usage, etc.
@@ -953,9 +1132,7 @@ class AgentBrowser:
                         final_message = agent_result.text
                     else:
                         # Fallback: try to get message if it exists as attribute
-                        final_message = getattr(
-                            agent_result, "message", "Task completed"
-                        )
+                        final_message = getattr(agent_result, "message", "Task completed")
                 elif isinstance(agent_result, str):
                     # String result (simple form)
                     final_message = agent_result
@@ -999,9 +1176,9 @@ class AgentBrowser:
                 if har_size > 100:  # HAR file should have some content
                     console.print(f" [dim]har saved: {har_size} bytes[/dim]")
                 else:
-                    console.print(f" [dim]har file exists but appears empty[/dim]")
+                    console.print(" [dim]har file exists but appears empty[/dim]")
             else:
-                console.print(f" [dim]har not saved[/dim]")
+                console.print(" [dim]har not saved[/dim]")
 
         return result
 
@@ -1009,7 +1186,7 @@ class AgentBrowser:
         """Start agent execution with HAR recording. Returns HAR path when done."""
         self._start_time = get_timestamp()
 
-        console.print(f" [dim]agent mode starting...[/dim]")
+        console.print(" [dim]agent mode starting...[/dim]")
         console.print(f" [dim]━[/dim] [white]{self.run_id}[/white]")
         console.print(f" [dim]task[/dim]  [white]{self.prompt}[/white]")
         console.print()
@@ -1020,7 +1197,7 @@ class AgentBrowser:
             result = asyncio.run(self._run_with_har_capture())
 
             if result.get("success"):
-                console.print(f" [green]agent task completed[/green]")
+                console.print(" [green]agent task completed[/green]")
                 msg = result.get("message", "").strip()
                 if msg:
                     if len(msg) > 500:
@@ -1032,10 +1209,11 @@ class AgentBrowser:
 
         except Exception as e:
             console.print(f" [red]error: {e}[/red]")
+            console.print(f" [dim]{ERROR_CTA}[/dim]")
             result = {"error": str(e)}
 
         self._save_metadata(get_timestamp(), result)
-        console.print(f" [dim]metadata synced[/dim]")
+        console.print(" [dim]metadata synced[/dim]")
 
         return self.har_path
 
@@ -1048,7 +1226,7 @@ def run_agent_browser(
     browser_use_model: str = "bu-llm",
     stagehand_model: str = "openai/computer-use-preview-2025-03-11",
     agent_provider: str = "browser-use",
-    start_url: Optional[str] = None,
+    start_url: str | None = None,
 ) -> Path:
     """Run agent browser with HAR recording."""
     agent_browser = AgentBrowser(
