@@ -9,7 +9,7 @@ import json
 import os
 from datetime import datetime
 from pathlib import Path
-from typing import Optional, Dict, Any
+from typing import Any
 
 import httpx
 
@@ -43,28 +43,26 @@ class OpenCodeEngineer(BaseEngineer):
         # Pop OpenCode-specific kwargs before passing to parent class
         self.opencode_provider = kwargs.pop("opencode_provider", "anthropic")
         self.opencode_model = kwargs.pop("opencode_model", "claude-sonnet-4-5")
-        
+
         super().__init__(*args, **kwargs)
-        
+
         # Override UI with OpenCode-specific version
         self.opencode_ui = OpenCodeUI(verbose=kwargs.get("verbose", True))
         self.ui = self.opencode_ui  # Ensure base class uses our specialized UI
-        self._last_error: Optional[str] = None
-        self._session_id: Optional[str] = None
+        self._last_error: str | None = None
+        self._session_id: str | None = None
         self._last_event_time = 0.0
 
-    async def analyze_and_generate(self) -> Optional[Dict[str, Any]]:
+    async def analyze_and_generate(self) -> dict[str, Any] | None:
         """Run the reverse engineering analysis with OpenCode."""
-        self.opencode_ui.header(self.run_id, self.prompt, self.opencode_model)
+        self.opencode_ui.header(self.run_id, self.prompt, self.opencode_model, self.sdk)
         self.opencode_ui.start_analysis()
 
         # Save the prompt to messages
         self.message_store.save_prompt(self._build_analysis_prompt())
 
         try:
-            async with httpx.AsyncClient(
-                base_url=self.BASE_URL, timeout=600.0
-            ) as client:
+            async with httpx.AsyncClient(base_url=self.BASE_URL, timeout=600.0) as client:
                 # Health check
                 try:
                     health_r = await client.get("/global/health")
@@ -73,9 +71,7 @@ class OpenCodeEngineer(BaseEngineer):
                     self.opencode_ui.health_check(health)
                 except Exception as e:
                     debug_log(f"Health check failed: {e}")
-                    self.opencode_ui.error(
-                        f"OpenCode server not responding. Is it running on {self.BASE_URL}?"
-                    )
+                    self.opencode_ui.error(f"OpenCode server not responding. Is it running on {self.BASE_URL}?")
                     return None
 
                 # Create a new session
@@ -105,15 +101,13 @@ class OpenCodeEngineer(BaseEngineer):
                     "parts": [{"type": "text", "text": self._build_analysis_prompt()}],
                 }
 
-                prompt_r = await client.post(
-                    f"/session/{self._session_id}/message", json=prompt_body
-                )
+                prompt_r = await client.post(f"/session/{self._session_id}/message", json=prompt_body)
                 prompt_r.raise_for_status()
 
                 # Wait for events to complete
                 try:
                     await asyncio.wait_for(event_task, timeout=600.0)
-                except asyncio.TimeoutError:
+                except TimeoutError:
                     self._last_error = "Session timed out (10 min)"
                     self.opencode_ui.error(self._last_error)
 
@@ -127,16 +121,12 @@ class OpenCodeEngineer(BaseEngineer):
                     return None
 
             # Success
-            script_path = str(self.scripts_dir / "api_client.py")
+            script_path = str(self.scripts_dir / self._get_client_filename())
 
             # Fetch actual provider and model used from session messages
             try:
-                async with httpx.AsyncClient(
-                    base_url=self.BASE_URL, timeout=10.0
-                ) as client:
-                    messages_r = await client.get(
-                        f"/session/{self._session_id}/message"
-                    )
+                async with httpx.AsyncClient(base_url=self.BASE_URL, timeout=10.0) as client:
+                    messages_r = await client.get(f"/session/{self._session_id}/message")
                     if messages_r.status_code == 200:
                         messages = messages_r.json()
                         # Find first assistant message with provider/model info
@@ -153,9 +143,10 @@ class OpenCodeEngineer(BaseEngineer):
 
             # Show session summary before success message
             self.opencode_ui.session_summary(self.usage_metadata)
-            self.opencode_ui.success(script_path)
+            local_path = str(self.local_scripts_dir / self._get_client_filename()) if self.local_scripts_dir else None
+            self.opencode_ui.success(script_path, local_path)
 
-            result_data: Dict[str, Any] = {
+            result_data: dict[str, Any] = {
                 "script_path": script_path,
                 "usage": self.usage_metadata,
                 "session_id": self._session_id,
@@ -165,9 +156,7 @@ class OpenCodeEngineer(BaseEngineer):
 
         except httpx.ConnectError:
             self.opencode_ui.error("Connection error")
-            self.opencode_ui.console.print(
-                "\n[dim]Make sure OpenCode is running: opencode[/dim]"
-            )
+            self.opencode_ui.console.print("\n[dim]Make sure OpenCode is running: opencode[/dim]")
             self.message_store.save_error("Connection error")
             return None
 
@@ -190,7 +179,7 @@ class OpenCodeEngineer(BaseEngineer):
         self.opencode_ui.start_streaming()
 
         try:
-            debug_log(f"Connecting to GET /event")
+            debug_log("Connecting to GET /event")
             async with client.stream("GET", "/event", timeout=None) as response:
                 debug_log(f"Event stream connected, status={response.status_code}")
                 async for line in response.aiter_lines():
@@ -215,6 +204,19 @@ class OpenCodeEngineer(BaseEngineer):
                     try:
                         data = json.loads(line_data)
                     except json.JSONDecodeError as e:
+                        error_str = str(e)
+                        # Check if this is a buffer size error
+                        if "buffer size" in error_str.lower() or "1048576" in error_str or "exceeded maximum buffer" in error_str.lower():
+                            debug_log(f"Buffer size error detected: {e}")
+                            self._last_error = (
+                                "Screenshot too large (exceeds 1MB limit). Try element-specific screenshots instead of full-page screenshots."
+                            )
+                            self.opencode_ui.error(self._last_error)
+                            self.opencode_ui.console.print(
+                                "[dim]Tip: Use browser_snapshot() for page structure or take smaller, element-specific screenshots.[/dim]"
+                            )
+                            # Don't continue - this is a fatal error for the session
+                            return
                         debug_log(f"JSON decode error: {e}, data: {line_data[:100]}")
                         continue
 
@@ -229,9 +231,7 @@ class OpenCodeEngineer(BaseEngineer):
 
                     elif event_type == "session.idle":
                         event_sid = properties.get("sessionID")
-                        debug_log(
-                            f"session.idle: sessionID={event_sid}, our session={self._session_id}"
-                        )
+                        debug_log(f"session.idle: sessionID={event_sid}, our session={self._session_id}")
                         if event_sid == self._session_id:
                             debug_log("Our session is idle, returning!")
                             self.opencode_ui.session_status("idle")
@@ -241,9 +241,7 @@ class OpenCodeEngineer(BaseEngineer):
                         event_sid = properties.get("sessionID")
                         status = properties.get("status", {})
                         status_type = status.get("type", "idle")
-                        debug_log(
-                            f"session.status: sessionID={event_sid}, status={status_type}"
-                        )
+                        debug_log(f"session.status: sessionID={event_sid}, status={status_type}")
                         if event_sid == self._session_id:
                             if status_type == "retry":
                                 attempt = status.get("attempt", 1)
@@ -263,9 +261,7 @@ class OpenCodeEngineer(BaseEngineer):
                         perm_type = properties.get("type", "")
                         perm_title = properties.get("title", "")
 
-                        debug_log(
-                            f"permission.updated: id={permission_id}, type={perm_type}, title={perm_title}"
-                        )
+                        debug_log(f"permission.updated: id={permission_id}, type={perm_type}, title={perm_title}")
 
                         if perm_session == self._session_id and permission_id:
                             # Show permission request in UI
@@ -277,13 +273,9 @@ class OpenCodeEngineer(BaseEngineer):
                             try:
                                 perm_response = await client.post(
                                     f"/session/{self._session_id}/permissions/{permission_id}",
-                                    json={
-                                        "response": "always"
-                                    },  # "once", "always", or "reject"
-                                )
-                                debug_log(
-                                    f"Permission response: {perm_response.status_code}"
+                                    json={"response": "always"},  # "once", "always", or "reject"
                                 )
+                                debug_log(f"Permission response: {perm_response.status_code}")
                                 if perm_response.status_code == 200:
                                     self.opencode_ui.permission_approved(perm_type)
                             except Exception as pe:
@@ -318,9 +310,7 @@ class OpenCodeEngineer(BaseEngineer):
                     elif event_type == "session.error":
                         event_sid = properties.get("sessionID")
                         if event_sid and event_sid != self._session_id:
-                            debug_log(
-                                f"session.error for other session {event_sid}, ignoring"
-                            )
+                            debug_log(f"session.error for other session {event_sid}, ignoring")
                             continue
 
                         error_obj = properties.get("error", {})
@@ -342,14 +332,8 @@ class OpenCodeEngineer(BaseEngineer):
                             elif error_name == "MessageAbortedError":
                                 self._last_error = "Aborted"
                             else:
-                                msg = (
-                                    error_data.get("message", "")
-                                    if isinstance(error_data, dict)
-                                    else str(error_data)
-                                )
-                                self._last_error = (
-                                    f"{error_name}: {msg}" if msg else error_name
-                                )
+                                msg = error_data.get("message", "") if isinstance(error_data, dict) else str(error_data)
+                                self._last_error = f"{error_name}: {msg}" if msg else error_name
                         else:
                             self._last_error = str(error_obj)
 
@@ -376,9 +360,7 @@ class OpenCodeEngineer(BaseEngineer):
 
         if part_type == "text":
             text = part.get("text", "")
-            debug_log(
-                f"Handling text part: id={part_id}, delta={'yes' if delta else 'no'}, len={len(text)}"
-            )
+            debug_log(f"Handling text part: id={part_id}, delta={'yes' if delta else 'no'}, len={len(text)}")
             # Use delta for incremental updates if available
             self.opencode_ui.update_text(text, delta)
 
@@ -392,9 +374,7 @@ class OpenCodeEngineer(BaseEngineer):
             state = part.get("state", {})
             status = state.get("status")
 
-            debug_log(
-                f"Handling tool part: id={part_id}, tool={tool_name}, status={status}"
-            )
+            debug_log(f"Handling tool part: id={part_id}, tool={tool_name}, status={status}")
 
             if status == "running" and part_id not in seen_parts:
                 seen_parts.add(part_id)
@@ -441,21 +421,11 @@ class OpenCodeEngineer(BaseEngineer):
             else:
                 cost = api_cost
 
-            self.usage_metadata["input_tokens"] = self.usage_metadata.get(
-                "input_tokens", 0
-            ) + input_tokens
-            self.usage_metadata["output_tokens"] = self.usage_metadata.get(
-                "output_tokens", 0
-            ) + output_tokens
-            self.usage_metadata["reasoning_tokens"] = self.usage_metadata.get(
-                "reasoning_tokens", 0
-            ) + reasoning_tokens
-            self.usage_metadata["cache_read_tokens"] = self.usage_metadata.get(
-                "cache_read_tokens", 0
-            ) + cache_read
-            self.usage_metadata["cache_creation_tokens"] = self.usage_metadata.get(
-                "cache_creation_tokens", 0
-            ) + cache_write
+            self.usage_metadata["input_tokens"] = self.usage_metadata.get("input_tokens", 0) + input_tokens
+            self.usage_metadata["output_tokens"] = self.usage_metadata.get("output_tokens", 0) + output_tokens
+            self.usage_metadata["reasoning_tokens"] = self.usage_metadata.get("reasoning_tokens", 0) + reasoning_tokens
+            self.usage_metadata["cache_read_tokens"] = self.usage_metadata.get("cache_read_tokens", 0) + cache_read
+            self.usage_metadata["cache_creation_tokens"] = self.usage_metadata.get("cache_creation_tokens", 0) + cache_write
             self.usage_metadata["cost"] = self.usage_metadata.get("cost", 0) + cost
 
 
@@ -463,13 +433,13 @@ def run_opencode_engineering(
     run_id: str,
     har_path: Path,
     prompt: str,
-    model: Optional[str] = None,
-    additional_instructions: Optional[str] = None,
-    output_dir: Optional[str] = None,
+    model: str | None = None,
+    additional_instructions: str | None = None,
+    output_dir: str | None = None,
     verbose: bool = True,
-    opencode_provider: Optional[str] = None,
-    opencode_model: Optional[str] = None,
-) -> Optional[Dict[str, Any]]:
+    opencode_provider: str | None = None,
+    opencode_model: str | None = None,
+) -> dict[str, Any] | None:
     """Synchronous wrapper for OpenCode reverse engineering."""
     engineer = OpenCodeEngineer(
         run_id=run_id,
